# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import (
    price_tracking_stop_params,
    price_tracking_start_params,
    price_tracking_get_history_params,
    price_tracking_list_subscriptions_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.subscription import Subscription
from ..types.price_history import PriceHistory
from ..types.paginated_subscriptions import PaginatedSubscriptions

__all__ = ["PriceTrackingResource", "AsyncPriceTrackingResource"]


class PriceTrackingResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PriceTrackingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/channel3-ai/sdk-python#accessing-raw-response-data-eg-headers
        """
        return PriceTrackingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PriceTrackingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/channel3-ai/sdk-python#with_streaming_response
        """
        return PriceTrackingResourceWithStreamingResponse(self)

    def get_history(
        self,
        canonical_product_id: str,
        *,
        days: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PriceHistory:
        """
        Get price history for a canonical product.

        Args:
          days: Number of days of history to fetch (max 30)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not canonical_product_id:
            raise ValueError(
                f"Expected a non-empty value for `canonical_product_id` but received {canonical_product_id!r}"
            )
        return self._get(
            f"/v0/price-tracking/history/{canonical_product_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"days": days}, price_tracking_get_history_params.PriceTrackingGetHistoryParams),
            ),
            cast_to=PriceHistory,
        )

    def list_subscriptions(
        self,
        *,
        limit: int | Omit = omit,
        page_token: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaginatedSubscriptions:
        """
        List your active price tracking subscriptions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v0/price-tracking/subscriptions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page_token": page_token,
                    },
                    price_tracking_list_subscriptions_params.PriceTrackingListSubscriptionsParams,
                ),
            ),
            cast_to=PaginatedSubscriptions,
        )

    def start(
        self,
        *,
        canonical_product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Subscription:
        """
        Start tracking prices for a canonical product.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v0/price-tracking/start",
            body=maybe_transform(
                {"canonical_product_id": canonical_product_id}, price_tracking_start_params.PriceTrackingStartParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    def stop(
        self,
        *,
        canonical_product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Subscription:
        """
        Stop tracking prices for a canonical product.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v0/price-tracking/stop",
            body=maybe_transform(
                {"canonical_product_id": canonical_product_id}, price_tracking_stop_params.PriceTrackingStopParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )


class AsyncPriceTrackingResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPriceTrackingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/channel3-ai/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPriceTrackingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPriceTrackingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/channel3-ai/sdk-python#with_streaming_response
        """
        return AsyncPriceTrackingResourceWithStreamingResponse(self)

    async def get_history(
        self,
        canonical_product_id: str,
        *,
        days: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PriceHistory:
        """
        Get price history for a canonical product.

        Args:
          days: Number of days of history to fetch (max 30)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not canonical_product_id:
            raise ValueError(
                f"Expected a non-empty value for `canonical_product_id` but received {canonical_product_id!r}"
            )
        return await self._get(
            f"/v0/price-tracking/history/{canonical_product_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"days": days}, price_tracking_get_history_params.PriceTrackingGetHistoryParams
                ),
            ),
            cast_to=PriceHistory,
        )

    async def list_subscriptions(
        self,
        *,
        limit: int | Omit = omit,
        page_token: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaginatedSubscriptions:
        """
        List your active price tracking subscriptions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v0/price-tracking/subscriptions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "page_token": page_token,
                    },
                    price_tracking_list_subscriptions_params.PriceTrackingListSubscriptionsParams,
                ),
            ),
            cast_to=PaginatedSubscriptions,
        )

    async def start(
        self,
        *,
        canonical_product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Subscription:
        """
        Start tracking prices for a canonical product.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v0/price-tracking/start",
            body=await async_maybe_transform(
                {"canonical_product_id": canonical_product_id}, price_tracking_start_params.PriceTrackingStartParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    async def stop(
        self,
        *,
        canonical_product_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Subscription:
        """
        Stop tracking prices for a canonical product.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v0/price-tracking/stop",
            body=await async_maybe_transform(
                {"canonical_product_id": canonical_product_id}, price_tracking_stop_params.PriceTrackingStopParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )


class PriceTrackingResourceWithRawResponse:
    def __init__(self, price_tracking: PriceTrackingResource) -> None:
        self._price_tracking = price_tracking

        self.get_history = to_raw_response_wrapper(
            price_tracking.get_history,
        )
        self.list_subscriptions = to_raw_response_wrapper(
            price_tracking.list_subscriptions,
        )
        self.start = to_raw_response_wrapper(
            price_tracking.start,
        )
        self.stop = to_raw_response_wrapper(
            price_tracking.stop,
        )


class AsyncPriceTrackingResourceWithRawResponse:
    def __init__(self, price_tracking: AsyncPriceTrackingResource) -> None:
        self._price_tracking = price_tracking

        self.get_history = async_to_raw_response_wrapper(
            price_tracking.get_history,
        )
        self.list_subscriptions = async_to_raw_response_wrapper(
            price_tracking.list_subscriptions,
        )
        self.start = async_to_raw_response_wrapper(
            price_tracking.start,
        )
        self.stop = async_to_raw_response_wrapper(
            price_tracking.stop,
        )


class PriceTrackingResourceWithStreamingResponse:
    def __init__(self, price_tracking: PriceTrackingResource) -> None:
        self._price_tracking = price_tracking

        self.get_history = to_streamed_response_wrapper(
            price_tracking.get_history,
        )
        self.list_subscriptions = to_streamed_response_wrapper(
            price_tracking.list_subscriptions,
        )
        self.start = to_streamed_response_wrapper(
            price_tracking.start,
        )
        self.stop = to_streamed_response_wrapper(
            price_tracking.stop,
        )


class AsyncPriceTrackingResourceWithStreamingResponse:
    def __init__(self, price_tracking: AsyncPriceTrackingResource) -> None:
        self._price_tracking = price_tracking

        self.get_history = async_to_streamed_response_wrapper(
            price_tracking.get_history,
        )
        self.list_subscriptions = async_to_streamed_response_wrapper(
            price_tracking.list_subscriptions,
        )
        self.start = async_to_streamed_response_wrapper(
            price_tracking.start,
        )
        self.stop = async_to_streamed_response_wrapper(
            price_tracking.stop,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr
from .redirect_mode import RedirectMode

__all__ = ["ProductRetrieveParams"]


class ProductRetrieveParams(TypedDict, total=False):
    redirect_mode: Optional[RedirectMode]
    """
    "price" redirects to the product page with the lowest price "commission"
    redirects to the product page with the highest commission rate "brand" redirects
    to the brand's product page
    """

    website_ids: Optional[SequenceNotStr[str]]
    """
    Optional list of website IDs to constrain the buy URL to, relevant if multiple
    merchants exist
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import Channel3Error, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import brands, enrich, search, products, websites, price_tracking
    from .resources.brands import BrandsResource, AsyncBrandsResource
    from .resources.enrich import EnrichResource, AsyncEnrichResource
    from .resources.search import SearchResource, AsyncSearchResource
    from .resources.products import ProductsResource, AsyncProductsResource
    from .resources.websites import WebsitesResource, AsyncWebsitesResource
    from .resources.price_tracking import PriceTrackingResource, AsyncPriceTrackingResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Channel3",
    "AsyncChannel3",
    "Client",
    "AsyncClient",
]


class Channel3(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Channel3 client instance.

        This automatically infers the `api_key` argument from the `CHANNEL3_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("CHANNEL3_API_KEY")
        if api_key is None:
            raise Channel3Error(
                "The api_key client option must be set either by passing api_key to the client or by setting the CHANNEL3_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("CHANNEL3_BASE_URL")
        if base_url is None:
            base_url = f"https://api.trychannel3.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def search(self) -> SearchResource:
        from .resources.search import SearchResource

        return SearchResource(self)

    @cached_property
    def products(self) -> ProductsResource:
        from .resources.products import ProductsResource

        return ProductsResource(self)

    @cached_property
    def brands(self) -> BrandsResource:
        from .resources.brands import BrandsResource

        return BrandsResource(self)

    @cached_property
    def websites(self) -> WebsitesResource:
        from .resources.websites import WebsitesResource

        return WebsitesResource(self)

    @cached_property
    def enrich(self) -> EnrichResource:
        from .resources.enrich import EnrichResource

        return EnrichResource(self)

    @cached_property
    def price_tracking(self) -> PriceTrackingResource:
        from .resources.price_tracking import PriceTrackingResource

        return PriceTrackingResource(self)

    @cached_property
    def with_raw_response(self) -> Channel3WithRawResponse:
        return Channel3WithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> Channel3WithStreamedResponse:
        return Channel3WithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncChannel3(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncChannel3 client instance.

        This automatically infers the `api_key` argument from the `CHANNEL3_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("CHANNEL3_API_KEY")
        if api_key is None:
            raise Channel3Error(
                "The api_key client option must be set either by passing api_key to the client or by setting the CHANNEL3_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("CHANNEL3_BASE_URL")
        if base_url is None:
            base_url = f"https://api.trychannel3.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def search(self) -> AsyncSearchResource:
        from .resources.search import AsyncSearchResource

        return AsyncSearchResource(self)

    @cached_property
    def products(self) -> AsyncProductsResource:
        from .resources.products import AsyncProductsResource

        return AsyncProductsResource(self)

    @cached_property
    def brands(self) -> AsyncBrandsResource:
        from .resources.brands import AsyncBrandsResource

        return AsyncBrandsResource(self)

    @cached_property
    def websites(self) -> AsyncWebsitesResource:
        from .resources.websites import AsyncWebsitesResource

        return AsyncWebsitesResource(self)

    @cached_property
    def enrich(self) -> AsyncEnrichResource:
        from .resources.enrich import AsyncEnrichResource

        return AsyncEnrichResource(self)

    @cached_property
    def price_tracking(self) -> AsyncPriceTrackingResource:
        from .resources.price_tracking import AsyncPriceTrackingResource

        return AsyncPriceTrackingResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncChannel3WithRawResponse:
        return AsyncChannel3WithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChannel3WithStreamedResponse:
        return AsyncChannel3WithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class Channel3WithRawResponse:
    _client: Channel3

    def __init__(self, client: Channel3) -> None:
        self._client = client

    @cached_property
    def search(self) -> search.SearchResourceWithRawResponse:
        from .resources.search import SearchResourceWithRawResponse

        return SearchResourceWithRawResponse(self._client.search)

    @cached_property
    def products(self) -> products.ProductsResourceWithRawResponse:
        from .resources.products import ProductsResourceWithRawResponse

        return ProductsResourceWithRawResponse(self._client.products)

    @cached_property
    def brands(self) -> brands.BrandsResourceWithRawResponse:
        from .resources.brands import BrandsResourceWithRawResponse

        return BrandsResourceWithRawResponse(self._client.brands)

    @cached_property
    def websites(self) -> websites.WebsitesResourceWithRawResponse:
        from .resources.websites import WebsitesResourceWithRawResponse

        return WebsitesResourceWithRawResponse(self._client.websites)

    @cached_property
    def enrich(self) -> enrich.EnrichResourceWithRawResponse:
        from .resources.enrich import EnrichResourceWithRawResponse

        return EnrichResourceWithRawResponse(self._client.enrich)

    @cached_property
    def price_tracking(self) -> price_tracking.PriceTrackingResourceWithRawResponse:
        from .resources.price_tracking import PriceTrackingResourceWithRawResponse

        return PriceTrackingResourceWithRawResponse(self._client.price_tracking)


class AsyncChannel3WithRawResponse:
    _client: AsyncChannel3

    def __init__(self, client: AsyncChannel3) -> None:
        self._client = client

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithRawResponse:
        from .resources.search import AsyncSearchResourceWithRawResponse

        return AsyncSearchResourceWithRawResponse(self._client.search)

    @cached_property
    def products(self) -> products.AsyncProductsResourceWithRawResponse:
        from .resources.products import AsyncProductsResourceWithRawResponse

        return AsyncProductsResourceWithRawResponse(self._client.products)

    @cached_property
    def brands(self) -> brands.AsyncBrandsResourceWithRawResponse:
        from .resources.brands import AsyncBrandsResourceWithRawResponse

        return AsyncBrandsResourceWithRawResponse(self._client.brands)

    @cached_property
    def websites(self) -> websites.AsyncWebsitesResourceWithRawResponse:
        from .resources.websites import AsyncWebsitesResourceWithRawResponse

        return AsyncWebsitesResourceWithRawResponse(self._client.websites)

    @cached_property
    def enrich(self) -> enrich.AsyncEnrichResourceWithRawResponse:
        from .resources.enrich import AsyncEnrichResourceWithRawResponse

        return AsyncEnrichResourceWithRawResponse(self._client.enrich)

    @cached_property
    def price_tracking(self) -> price_tracking.AsyncPriceTrackingResourceWithRawResponse:
        from .resources.price_tracking import AsyncPriceTrackingResourceWithRawResponse

        return AsyncPriceTrackingResourceWithRawResponse(self._client.price_tracking)


class Channel3WithStreamedResponse:
    _client: Channel3

    def __init__(self, client: Channel3) -> None:
        self._client = client

    @cached_property
    def search(self) -> search.SearchResourceWithStreamingResponse:
        from .resources.search import SearchResourceWithStreamingResponse

        return SearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def products(self) -> products.ProductsResourceWithStreamingResponse:
        from .resources.products import ProductsResourceWithStreamingResponse

        return ProductsResourceWithStreamingResponse(self._client.products)

    @cached_property
    def brands(self) -> brands.BrandsResourceWithStreamingResponse:
        from .resources.brands import BrandsResourceWithStreamingResponse

        return BrandsResourceWithStreamingResponse(self._client.brands)

    @cached_property
    def websites(self) -> websites.WebsitesResourceWithStreamingResponse:
        from .resources.websites import WebsitesResourceWithStreamingResponse

        return WebsitesResourceWithStreamingResponse(self._client.websites)

    @cached_property
    def enrich(self) -> enrich.EnrichResourceWithStreamingResponse:
        from .resources.enrich import EnrichResourceWithStreamingResponse

        return EnrichResourceWithStreamingResponse(self._client.enrich)

    @cached_property
    def price_tracking(self) -> price_tracking.PriceTrackingResourceWithStreamingResponse:
        from .resources.price_tracking import PriceTrackingResourceWithStreamingResponse

        return PriceTrackingResourceWithStreamingResponse(self._client.price_tracking)


class AsyncChannel3WithStreamedResponse:
    _client: AsyncChannel3

    def __init__(self, client: AsyncChannel3) -> None:
        self._client = client

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithStreamingResponse:
        from .resources.search import AsyncSearchResourceWithStreamingResponse

        return AsyncSearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def products(self) -> products.AsyncProductsResourceWithStreamingResponse:
        from .resources.products import AsyncProductsResourceWithStreamingResponse

        return AsyncProductsResourceWithStreamingResponse(self._client.products)

    @cached_property
    def brands(self) -> brands.AsyncBrandsResourceWithStreamingResponse:
        from .resources.brands import AsyncBrandsResourceWithStreamingResponse

        return AsyncBrandsResourceWithStreamingResponse(self._client.brands)

    @cached_property
    def websites(self) -> websites.AsyncWebsitesResourceWithStreamingResponse:
        from .resources.websites import AsyncWebsitesResourceWithStreamingResponse

        return AsyncWebsitesResourceWithStreamingResponse(self._client.websites)

    @cached_property
    def enrich(self) -> enrich.AsyncEnrichResourceWithStreamingResponse:
        from .resources.enrich import AsyncEnrichResourceWithStreamingResponse

        return AsyncEnrichResourceWithStreamingResponse(self._client.enrich)

    @cached_property
    def price_tracking(self) -> price_tracking.AsyncPriceTrackingResourceWithStreamingResponse:
        from .resources.price_tracking import AsyncPriceTrackingResourceWithStreamingResponse

        return AsyncPriceTrackingResourceWithStreamingResponse(self._client.price_tracking)


Client = Channel3

AsyncClient = AsyncChannel3



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PriceHistory", "History", "Statistics"]


class History(BaseModel):
    currency: str

    price: float

    timestamp: datetime


class Statistics(BaseModel):
    currency: str

    current_price: float

    current_status: Literal["low", "typical", "high"]

    max_price: float

    mean: float

    min_price: float

    std_dev: float


class PriceHistory(BaseModel):
    canonical_product_id: str

    history: Optional[List[History]] = None

    product_title: Optional[str] = None

    statistics: Optional[Statistics] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["SearchFilterPriceParam"]


class SearchFilterPriceParam(TypedDict, total=False):
    """Price filter. Values are inclusive."""

    max_price: Optional[float]
    """Maximum price, in dollars and cents"""

    min_price: Optional[float]
    """Minimum price, in dollars and cents"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import enrich_enrich_url_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.product_detail import ProductDetail

__all__ = ["EnrichResource", "AsyncEnrichResource"]


class EnrichResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EnrichResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/channel3-ai/sdk-python#accessing-raw-response-data-eg-headers
        """
        return EnrichResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EnrichResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/channel3-ai/sdk-python#with_streaming_response
        """
        return EnrichResourceWithStreamingResponse(self)

    def enrich_url(
        self,
        *,
        url: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductDetail:
        """
        Search by product URL, get back full product information from Channel3’s product
        database.

        Args:
          url: The URL of the product to enrich

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v0/enrich",
            body=maybe_transform({"url": url}, enrich_enrich_url_params.EnrichEnrichURLParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductDetail,
        )


class AsyncEnrichResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEnrichResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/channel3-ai/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEnrichResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEnrichResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/channel3-ai/sdk-python#with_streaming_response
        """
        return AsyncEnrichResourceWithStreamingResponse(self)

    async def enrich_url(
        self,
        *,
        url: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductDetail:
        """
        Search by product URL, get back full product information from Channel3’s product
        database.

        Args:
          url: The URL of the product to enrich

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v0/enrich",
            body=await async_maybe_transform({"url": url}, enrich_enrich_url_params.EnrichEnrichURLParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductDetail,
        )


class EnrichResourceWithRawResponse:
    def __init__(self, enrich: EnrichResource) -> None:
        self._enrich = enrich

        self.enrich_url = to_raw_response_wrapper(
            enrich.enrich_url,
        )


class AsyncEnrichResourceWithRawResponse:
    def __init__(self, enrich: AsyncEnrichResource) -> None:
        self._enrich = enrich

        self.enrich_url = async_to_raw_response_wrapper(
            enrich.enrich_url,
        )


class EnrichResourceWithStreamingResponse:
    def __init__(self, enrich: EnrichResource) -> None:
        self._enrich = enrich

        self.enrich_url = to_streamed_response_wrapper(
            enrich.enrich_url,
        )


class AsyncEnrichResourceWithStreamingResponse:
    def __init__(self, enrich: AsyncEnrichResource) -> None:
        self._enrich = enrich

        self.enrich_url = async_to_streamed_response_wrapper(
            enrich.enrich_url,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from channel3_sdk import Channel3, AsyncChannel3
from channel3_sdk.types import (
    PriceHistory,
    Subscription,
    PaginatedSubscriptions,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPriceTracking:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_history(self, client: Channel3) -> None:
        price_tracking = client.price_tracking.get_history(
            canonical_product_id="canonical_product_id",
        )
        assert_matches_type(PriceHistory, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_history_with_all_params(self, client: Channel3) -> None:
        price_tracking = client.price_tracking.get_history(
            canonical_product_id="canonical_product_id",
            days=1,
        )
        assert_matches_type(PriceHistory, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_history(self, client: Channel3) -> None:
        response = client.price_tracking.with_raw_response.get_history(
            canonical_product_id="canonical_product_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_tracking = response.parse()
        assert_matches_type(PriceHistory, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_history(self, client: Channel3) -> None:
        with client.price_tracking.with_streaming_response.get_history(
            canonical_product_id="canonical_product_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_tracking = response.parse()
            assert_matches_type(PriceHistory, price_tracking, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_history(self, client: Channel3) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `canonical_product_id` but received ''"):
            client.price_tracking.with_raw_response.get_history(
                canonical_product_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_subscriptions(self, client: Channel3) -> None:
        price_tracking = client.price_tracking.list_subscriptions()
        assert_matches_type(PaginatedSubscriptions, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_subscriptions_with_all_params(self, client: Channel3) -> None:
        price_tracking = client.price_tracking.list_subscriptions(
            limit=1,
            page_token="page_token",
        )
        assert_matches_type(PaginatedSubscriptions, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_subscriptions(self, client: Channel3) -> None:
        response = client.price_tracking.with_raw_response.list_subscriptions()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_tracking = response.parse()
        assert_matches_type(PaginatedSubscriptions, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_subscriptions(self, client: Channel3) -> None:
        with client.price_tracking.with_streaming_response.list_subscriptions() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_tracking = response.parse()
            assert_matches_type(PaginatedSubscriptions, price_tracking, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_start(self, client: Channel3) -> None:
        price_tracking = client.price_tracking.start(
            canonical_product_id="canonical_product_id",
        )
        assert_matches_type(Subscription, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_start(self, client: Channel3) -> None:
        response = client.price_tracking.with_raw_response.start(
            canonical_product_id="canonical_product_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_tracking = response.parse()
        assert_matches_type(Subscription, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_start(self, client: Channel3) -> None:
        with client.price_tracking.with_streaming_response.start(
            canonical_product_id="canonical_product_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_tracking = response.parse()
            assert_matches_type(Subscription, price_tracking, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_stop(self, client: Channel3) -> None:
        price_tracking = client.price_tracking.stop(
            canonical_product_id="canonical_product_id",
        )
        assert_matches_type(Subscription, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_stop(self, client: Channel3) -> None:
        response = client.price_tracking.with_raw_response.stop(
            canonical_product_id="canonical_product_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_tracking = response.parse()
        assert_matches_type(Subscription, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_stop(self, client: Channel3) -> None:
        with client.price_tracking.with_streaming_response.stop(
            canonical_product_id="canonical_product_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_tracking = response.parse()
            assert_matches_type(Subscription, price_tracking, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncPriceTracking:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_history(self, async_client: AsyncChannel3) -> None:
        price_tracking = await async_client.price_tracking.get_history(
            canonical_product_id="canonical_product_id",
        )
        assert_matches_type(PriceHistory, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_history_with_all_params(self, async_client: AsyncChannel3) -> None:
        price_tracking = await async_client.price_tracking.get_history(
            canonical_product_id="canonical_product_id",
            days=1,
        )
        assert_matches_type(PriceHistory, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_history(self, async_client: AsyncChannel3) -> None:
        response = await async_client.price_tracking.with_raw_response.get_history(
            canonical_product_id="canonical_product_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_tracking = await response.parse()
        assert_matches_type(PriceHistory, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_history(self, async_client: AsyncChannel3) -> None:
        async with async_client.price_tracking.with_streaming_response.get_history(
            canonical_product_id="canonical_product_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_tracking = await response.parse()
            assert_matches_type(PriceHistory, price_tracking, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_history(self, async_client: AsyncChannel3) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `canonical_product_id` but received ''"):
            await async_client.price_tracking.with_raw_response.get_history(
                canonical_product_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_subscriptions(self, async_client: AsyncChannel3) -> None:
        price_tracking = await async_client.price_tracking.list_subscriptions()
        assert_matches_type(PaginatedSubscriptions, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_subscriptions_with_all_params(self, async_client: AsyncChannel3) -> None:
        price_tracking = await async_client.price_tracking.list_subscriptions(
            limit=1,
            page_token="page_token",
        )
        assert_matches_type(PaginatedSubscriptions, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_subscriptions(self, async_client: AsyncChannel3) -> None:
        response = await async_client.price_tracking.with_raw_response.list_subscriptions()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_tracking = await response.parse()
        assert_matches_type(PaginatedSubscriptions, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_subscriptions(self, async_client: AsyncChannel3) -> None:
        async with async_client.price_tracking.with_streaming_response.list_subscriptions() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_tracking = await response.parse()
            assert_matches_type(PaginatedSubscriptions, price_tracking, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_start(self, async_client: AsyncChannel3) -> None:
        price_tracking = await async_client.price_tracking.start(
            canonical_product_id="canonical_product_id",
        )
        assert_matches_type(Subscription, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_start(self, async_client: AsyncChannel3) -> None:
        response = await async_client.price_tracking.with_raw_response.start(
            canonical_product_id="canonical_product_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_tracking = await response.parse()
        assert_matches_type(Subscription, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_start(self, async_client: AsyncChannel3) -> None:
        async with async_client.price_tracking.with_streaming_response.start(
            canonical_product_id="canonical_product_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_tracking = await response.parse()
            assert_matches_type(Subscription, price_tracking, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_stop(self, async_client: AsyncChannel3) -> None:
        price_tracking = await async_client.price_tracking.stop(
            canonical_product_id="canonical_product_id",
        )
        assert_matches_type(Subscription, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_stop(self, async_client: AsyncChannel3) -> None:
        response = await async_client.price_tracking.with_raw_response.stop(
            canonical_product_id="canonical_product_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_tracking = await response.parse()
        assert_matches_type(Subscription, price_tracking, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_stop(self, async_client: AsyncChannel3) -> None:
        async with async_client.price_tracking.with_streaming_response.stop(
            canonical_product_id="canonical_product_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_tracking = await response.parse()
            assert_matches_type(Subscription, price_tracking, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .price import Price
from .variant import Variant
from .._models import BaseModel
from .availability_status import AvailabilityStatus

__all__ = ["Product", "Image"]


class Image(BaseModel):
    """Product image with metadata"""

    url: str

    alt_text: Optional[str] = None

    is_main_image: Optional[bool] = None

    photo_quality: Optional[Literal["professional", "ugc", "poor"]] = None
    """
    Photo quality classification for API responses. Note: This enum is decoupled
    from internal ImageIntelligence types as they may diverge.
    """

    shot_type: Optional[
        Literal[
            "hero",
            "lifestyle",
            "on_model",
            "detail",
            "scale_reference",
            "angle_view",
            "flat_lay",
            "in_use",
            "packaging",
            "size_chart",
            "color_swatch",
            "product_information",
            "merchant_information",
        ]
    ] = None
    """
    Product image type classification for API responses. Note: This enum is
    decoupled from internal ImageIntelligence types as they may diverge.
    """


class Product(BaseModel):
    """A search result that includes product details and a relevance score."""

    id: str

    availability: AvailabilityStatus

    image_url: str
    """Main product image (deprecated, use images field)"""

    price: Price

    score: int

    title: str

    url: str

    brand_id: Optional[str] = None

    brand_name: Optional[str] = None

    categories: Optional[List[str]] = None

    description: Optional[str] = None

    gender: Optional[Literal["male", "female", "unisex"]] = None

    image_urls: Optional[List[str]] = None
    """List of image URLs (deprecated, use images field)"""

    images: Optional[List[Image]] = None

    key_features: Optional[List[str]] = None

    materials: Optional[List[str]] = None

    variants: Optional[List[Variant]] = None
    """Simple variant list (deprecated, use variant_info for full variant details)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .subscription import Subscription

__all__ = ["PaginatedSubscriptions"]


class PaginatedSubscriptions(BaseModel):
    subscriptions: List[Subscription]

    next_page_token: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PriceTrackingGetHistoryParams"]


class PriceTrackingGetHistoryParams(TypedDict, total=False):
    days: int
    """Number of days of history to fetch (max 30)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Price"]


class Price(BaseModel):
    currency: str
    """The currency code of the product, like USD, EUR, GBP, etc."""

    price: float
    """The current price of the product, including any discounts."""

    compare_at_price: Optional[float] = None
    """The original price of the product before any discounts."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .redirect_mode import RedirectMode

__all__ = ["SearchConfigParam"]


class SearchConfigParam(TypedDict, total=False):
    """Configuration for a search request"""

    keyword_search_only: bool
    """If True, search will only use keyword search and not vector search.

    Keyword-only search is not supported with image input.
    """

    redirect_mode: Optional[RedirectMode]
    """
    "price" redirects to the product page with the lowest price "commission"
    redirects to the product page with the highest commission rate "brand" redirects
    to the brand's product page
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Subscription"]


class Subscription(BaseModel):
    canonical_product_id: str

    created_at: datetime

    subscription_status: Literal["active", "cancelled"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Website"]


class Website(BaseModel):
    id: str

    url: str

    best_commission_rate: Optional[float] = None
    """The maximum commission rate for the website, as a percentage"""
